package cz.uhk.iot.gui;

import cz.uhk.iot.gui.panel.ControlPanel;
import cz.uhk.iot.gui.panel.LogPanel;
import cz.uhk.iot.gui.panel.SerialPortConnectPanel;
import cz.uhk.iot.service.SecurityService;
import cz.uhk.iot.service.SerialCommunicationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import javax.swing.*;
import java.awt.*;


public class MainFrame extends JFrame {

    private Logger LOG = LoggerFactory.getLogger(MainFrame.class);

    private LogPanel logPanel;
    private ControlPanel controlPanel;

    @Autowired
    private SerialCommunicationService serialCommunicationService;

    @Autowired
    SecurityService securityService;

    public MainFrame() {
        super("Smart Home Gateway");
    }

    public void start() {
        LOG.info("Starting smart home gateway GUI");
        if (!serialCommunicationService.init(this)) {
            System.exit(-1);
        }
        setUpFrame();

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setSize(new Dimension(600, 400));

        setState(Frame.NORMAL);
        pack();
        setVisible(true);
    }

    private void setUpFrame() {
        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);

        GridBagConstraints c = new GridBagConstraints();

        c.weightx = 0.5;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        SerialPortConnectPanel serialPortConnectPanel = new SerialPortConnectPanel(serialCommunicationService);
        add(serialPortConnectPanel, c);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1;
        c.gridx = 1;
        c.gridy = 0;
        add(setUpWelcomePanel(), c);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 0.5;
        c.weighty = 2;
        c.gridx = 0;
        c.gridy = 1;
        logPanel = new LogPanel();
        add(logPanel, c);

        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 0.5;
        c.gridx = 1;
        c.gridy = 1;
        controlPanel = new ControlPanel(serialCommunicationService, securityService);
        add(controlPanel, c);
    }

    public void setTemperature(String temperature) {
        String[] tempData = temperature.split("_");
        if (tempData[0].equals("BT")) {
            controlPanel.getTemperaturePanel().setBluetoothTemp(tempData[2]);
        } else {
            controlPanel.getTemperaturePanel().setZigBeeTemp(tempData[2]);
        }
    }

    public void setStatusData(String data) {
        logPanel.addLog(data + "\n");
    }

    private JPanel setUpWelcomePanel() {
        JPanel panel = new JPanel();
        panel.setBackground(new Color(97, 255, 158));
        panel.setBorder(BorderFactory.createLineBorder(Color.black));
        JLabel appName = new JLabel("Ovládací panel");
        appName.setFont(new Font("San-Serif", Font.PLAIN, 16));
        appName.setHorizontalAlignment(SwingConstants.CENTER);
        panel.add(appName);
        return panel;
    }
}
