package cz.uhk.iot.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.tinyradius.packet.AccessRequest;
import org.tinyradius.packet.RadiusPacket;
import org.tinyradius.util.RadiusClient;

@Service
public class RadiusService {

    private RadiusClient radiusClient;

    private Logger LOG = LoggerFactory.getLogger(RadiusService.class);

    @Autowired
    public RadiusService(@Value("${gw.ip:127.0.0.1}") String ipAddress, @Value("${gw.secret:password}") String secret) {
        this.radiusClient = new RadiusClient(ipAddress, secret);
    }

    public boolean authenticate(String name, String password) {
        return sendAccessRequest(radiusClient, name, password);
    }

    private boolean sendAccessRequest(RadiusClient rc, String name, String password) {
        try {
            AccessRequest request = new AccessRequest(name, password);
            request.setAuthProtocol(AccessRequest.AUTH_CHAP);

            LOG.debug("Sending:\n" + request.toString());

            RadiusPacket reply = rc.authenticate(request);
            if (reply.getPacketType() == RadiusPacket.ACCESS_ACCEPT) {
                LOG.info("User '{}' was successfully authenticated", name);
                return true;
            }
        } catch (Exception e) {
            LOG.error("RADIUS authentication failed", e);
        }
        return false;
    }
}
