#define RX 7
#define TX 8
#define LED_PIN 9

#include <SoftwareSerial.h>
#include <MD5.h>
#include <sha256.h>

SoftwareSerial remote(TX, RX);
Sha256 sha256;

float temp = 0.0;
String username = "BluetoothNode";
String password = "passwordnode";
String hashPass = "";
boolean authorized = false;
boolean isAuthorizing = false;
long timeAuthStart = 0;
long timeTempStart = -10;
long timeAuthRequestStart = 0;
long authPeriod = 60 * 60; //1 hour
long tempPeriod = 10; //10 seconds
long authRequestPeriod = 20;
String authCRC = "";
String DELIMITER = "_";

void setup() {
  remote.begin(9600);
  remote.println("Arduino zapnuto.");
  pinMode(LED_PIN, OUTPUT);
  hashPass = hashMD5(hashSHA256(password));
  randomSeed(analogRead(0));
}

void loop() {
  handleRemoteData();
  if (authorized && getTime() > timeAuthStart + authPeriod) {
    authorized = false;
  }

  if (getTime() > timeTempStart + tempPeriod) { //aktualizace teploty jednou za 10 vteřin
    timeTempStart = getTime();
    temp = rightTemp();
  }

  if (getTime() > timeAuthRequestStart + authRequestPeriod) {
    isAuthorizing = false;
  }
}

void sendAuthRequest() {
  int randNum = getRandomNumber();
  authCRC = hashMD5(username + randNum);
  String authToken = String("auth" + DELIMITER + username + DELIMITER + hashPass + DELIMITER + randNum);
  remote.println(authToken);
  isAuthorizing = true;
  timeAuthRequestStart = getTime();
}

float rightTemp() {
  float temperature = 0.0;
  int sample;
  float sumSamples = 0.0;

  for (sample = 0; sample < 5; sample++) {
    temperature = ((float)analogRead(0) * 5.0 / 1024.0) - 0.5;
    temperature = temperature / 0.01;
    // měření každých 50ms
    delay(50);
    sumSamples = sumSamples + temperature;
  }
  temperature = sumSamples / 5.0;
  return temperature;
}

void sendTemp() {
  String tempData("temp" + DELIMITER + temp);
  remote.println(tempData);
}

void handleRemoteData() {
  byte remoteData;
  if (remote.available() > 0) {
    remoteData = remote.read();
    if (isAuthorizing && remoteData == 'A') {
      while (remote.available() == 0);
      String incomeCRC = remote.readString();
      incomeCRC = incomeCRC.substring(0, incomeCRC.indexOf('\r'));
      if (incomeCRC.equals(authCRC)) {
        authorized = true;
        timeAuthStart = getTime();
        isAuthorizing = false;
      }
    }
    else if (remoteData == '?') {
      if (!authorized && !isAuthorizing) {
        sendAuthRequest();
      }
      if (authorized) {
        sendTemp();
      }
    }
    else if (authorized) {
      switch (remoteData) {
        case 'p': {
            while (remote.available() == 0);
            String passChangeString = remote.readString();
            if (changePassword(passChangeString)) {
              remote.println("Password was changed.");
            } else {
              remote.println("Password change failed!");
            }
            break;
        } case '0':
          digitalWrite(LED_PIN, LOW);
          remote.println("Světlo bylo vypnuto.");
          break;
        case '1':
          digitalWrite(LED_PIN, HIGH);
          remote.println("Světlo bylo zapnuto.");
          break;
        case 'r':
          // čas od spuštění Arduina
          remote.print("Čas od spuštění Arduina: ");
          remote.print(getTime());
          remote.println(" vteřin.");
          break;
        case '\r':
          break;
        case '\n':
          break;
        default:
          remote.print("Neznámý příkaz: ");
          remote.println(remoteData);
      }
    } else {
      while (remote.available() == 0); //vyčištění bufferu
    }
  }
  delay(100);
}

boolean changePassword(String passChangeString) {
  int splitIndex = passChangeString.indexOf(DELIMITER);
  int endIndex = passChangeString.indexOf('\r');
  String oldPass = passChangeString.substring(0, splitIndex);
  String newPass = passChangeString.substring(splitIndex + 1, endIndex);
  if (oldPass.equals(password)) {
    password = newPass;
    hashPass = hashMD5(hashSHA256(password));
    authorized = false;
    return true;
  }
  return false;
}

int getTime() {
  return millis() / 1000;
}

String hashMD5(String input) {
  char charPass[input.length() + 1];
  input.toCharArray(charPass, input.length() + 1);
  unsigned char* hash = MD5::make_hash(charPass);
  char* md5str = MD5::make_digest(hash, 16);
  return md5str;
}

String hashSHA256(String input) {
  char encoded[64];
  sha256.init();
  sha256.print(input);
  uint8_t* result = sha256.result();
  String hash = "";
  for (int i = 0; i < 32; i++) {
    hash += "0123456789abcdef"[result[i] >> 4];
    hash += "0123456789abcdef"[result[i] & 0xf];
  }
  return hash;
}

int getRandomNumber() {
  return random(1000, 9999);
}
